input.easynewsletter_text {
  width: 96%;
  margin-right: 0;
}

#enl_actions{
  margin: 1em 0 2em 0;
  padding:0.5em;
  border-bottom: 1px solid #75AD0A;
}

#enl_actions li{
  display: inline;
  margin: 0 1em 0 0;
}

.portletNewsletterSubscriber div.field span.required::after {
    position: absolute;
}

.template-atct_edit.portaltype-enltemplate #archetypes-fieldname-body textarea{
    height: 850px;
    min-width: 800px;
    font-family: Courier New;
    font-size: 90%;
}
